//! \file sprite.c
#include<SDL.h>
#include<action.h>
#include<world.h>
#include<display.h>
#include<object.h>
#include<math.h>

#define _SPRITE_C_

/*! Internal representation of a sprite. It contains all the necesary
 information for a sprite to do its job.
 */
struct sprite{
    world world; 	//!<World where the sprite has been attached.
    display display;	//!<A display where the sprite has to draw himself.
    int x,y;		//!<The position.
    int w,h;		//!<The dimansions
    int lifetime;	//!<How many world steps this have to last.
    SDL_Surface * old;	//!<How looked the display before the blit.
    SDL_Surface * sprites;	//!<The actual graphic to put.
    int nsprites;	//!<Numver of sprite in sprites.(could be several arranged in some way)
    id id;		//!<The id of the object.Have to be uniqe.
    int step;		//!<Number of steps of life.
};

typedef struct sprite * sprite;
#include<sprite.h>

/*! Allocates the resorces needed by a sprite, loads the grapics. It must
 be called befor te use of any other function related to this sprite.
\return A ready for use spriute. 
\param  world The world wich is attached to.
\param  display A display to draw himself. 
\param  id The unique object id, needed for event passing.
\param  y Y position relative to te left top corner of the display.  
\param  x X position relative to te left top corner of the display.
\param w The width of tyhe sprite.
\param h The heigth of tyhe sprite.
\param spritef A graphic filename where the sprites are stored.
*/
sprite sprite_init(world world, display display, id id, int x, int y, int w, int h, int lifetime, char * spritef){
    sprite sprite=malloc(sizeof(struct spritef));
    sprite->world=world;
    sprite->display = display;
    sprite->x=x;
    sprite->y=y;
    sprite->w=w;
    sprite->h=h;
    sprite->lifetime=lifetime;
    sprite->id=id;
    sprite->step=0;
    sprite->sprite=SDL_LoadBMP(sprite);
    sprite->oldsprite=SDL_CreateSurface(w,h)
    sprite->nsprites=sprite->sprite->h/h;    
    if(sprite->sprite->h%h || sprite->sprite->w != w)
	printf("Error loading %s.\n",spritef);    
    return sprite;
};
/*!It unallocates the resources used by the prite.
\param A sprite.*/
void sprite_destroy(sprite sprite){
    free(sprite);
};

/*! It doesn't do anything.
\bug In fact, it doesn't have to do anything... All the work must be y sprite_show.
\param sprite A sprite.
*/
int sprite_step(sprite sprite){
    static SDL_Rect rect;
    if(sprite->lifetime<sprite->step)
	return 0;
    if(sprite->step){	
        rect.x=x;rect.y=y;rect.w=w;rect.h=h;
	display_get(sprite->display,sprite->oldsprite,&rect);
    }
    display_blit(sprite->display, sprite->sprite, 

    sprite->step++;
return 1;
};
/*! It checks if the sprite is still alive or active or running or being.
\return -0 Not running
-1 running
\param sprite A sprite.
*/
int sprite_alive(sprite sprite){
    return (sprite->lifetime>sprite->step);
};

/*! Draw the sprite in the display.
\param sprite A sprite.*/
int sprite_show(sprite sprite){
    display_drawpixel(sprite->display, sprite->x, sprite->y, sprite->color);
    return 1;
};

/*! Dummy */
int sprite_action(sprite sprite, SDL_Event * action){
    return 0;
};


