//! \file Implementacion of the functions to manipulkate a world ADT
#ifndef PUNE_WORLD
#define PUNE_WORLD

#include<stdlib.h>
#include<list.h>
#include<collide.h>
#include<display.h>
#include<object.h>
#include<SDL.h>

//start bugs
#define MAXTILES 5
//end bugs

#define _WORLD_C_

//! Internal representation of a world.
struct world
{
  int numobjects; //!< Number of all objects in the world.
  list_t all_objects; //!< List of all objects in the world.
  int numzombies;//!< Number of objects not active, waiting for deletion.
  list_t zombies;//!< List of objects not active, waiting for deletion.
  int numsleeping;//!< Number of all objects not active but still alive.
  list_t sleeping;//!< List of objects not active but still alive.
  int numrunning;//!< Number of all running objects in the world.
  list_t running;//!< List of all running objects in the world.
};

//void create_object(world, object, whichtile, whichplane);

typedef struct world *world;

/*! Initializes and allocates resources for a world. Must be called before
the use of any other operation on the world.
\return A ready to use world without any object in it.
*/
world
world_init ()
{
  world world;
  world = malloc (sizeof (struct world));
  world->numobjects = 0;
  world->numrunning = 0;
  world->numzombies = 0;
  INIT_LIST_HEAD (&world->all_objects);
  INIT_LIST_HEAD (&world->zombies);
  INIT_LIST_HEAD (&world->sleeping);
  INIT_LIST_HEAD (&world->running);

  return world;
};

/*! Destroy a world and every object on it. It frees the resources 
previously allocated.
\param world The world to be destroyed. 
*/
void world_destroy(world world){
    free(world);  // <------jeje
}

/*! Attaches an object to a world. The destruction of 
the object is up to the world.
\param world An initialized world.
\param obj An object to be attached to the world.
*/

void
world_attach_object (world world, object obj)
{
  world->numobjects++;
  world->numrunning++;
  list_add (&obj->all_objects, &world->all_objects);
  list_add (&obj->list, &world->running);
};

/*! Here is where the magic is done. It cycles for every object and 
ask everyone to do their things. It also delivers the actions or 
events that the objects eventualy send eachother.
\bug Realtime? expect a time? 
\param world A world.
\return If the worldhave some object running.
    - 0 Not running
    - 1 At least one object is running.
*/

int world_step (world world)
{
    list_t *objl;
    list_t *aux;
    object obj;
    static SDL_Event event;
    int ticks;

	ticks = SDL_GetTicks ();
	while (SDL_PollEvent (&event))
	        list_for_each (objl, &world->running) {
	    	    obj = list_entry (objl, struct object, list);
	    	    obj_action (obj, &event);
	        }
	list_for_each_safe (objl, aux, &world->running) {
	    obj = list_entry (objl, struct object, list);
	    obj_step (obj);
	    //SDL_Delay (1);
	    if (!obj_alive (obj)) {
		world->numrunning--;
		world->numzombies++;
		list_del (&obj->list);
		list_add (&obj->list, &world->zombies);
	    }
	}
	if (world->numzombies) {
	    obj = list_entry (world->zombies.next, struct object, list);
	    list_del (world->zombies.next);
	    list_del (&obj->all_objects);
	    world->numobjects--;
	    world->numzombies--;
	    object_destroy (obj);
	};
    
    //display_update(world_getdisplay(world, 0));

    return  world->numrunning;
};

#endif
